<?php
/*                                                                                                                 
Plugin Name: XHTML Content Negotiation
Plugin URI: http://rayofsolaris.net/blog/plugins/xhtml-content-negotiation-for-wordpress/
Description: Sends content as application/xhtml+xml based on the browser's ACCEPT request.  Ignores Internet Explorer which can't handle application/xhtml+xml.
Version: 1.3
Author: Samir Shah
Author URI: http://rayofsolaris.net/
*/

/*  Copyright 2009  Samir Shah  (email : samir[at]rayofsolaris.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function q_value($type, $accept_string) {
	if(preg_match("/".preg_quote($type,'/').";\s?q=([01]|0\.\d{1,3}|1\.0)/i",$accept_string,$matches)) {
		return (float)$matches[1];
	}
	/* Default is 1 */
	return 1;
}

function set_content_type($default) {
	$accept_string = isset($_SERVER['HTTP_ACCEPT']) ? $_SERVER['HTTP_ACCEPT'] : '';
	$request_uri = $_SERVER['REQUEST_URI'];
	$user_agent = $_SERVER['HTTP_USER_AGENT'];

	/* The types looked for */
	$type = array(
		'default'=>$default,
		'xhtml'=>'application/xhtml+xml',
		'any'=>'*/*'
	);

	/* If it is an admin page */
	if(preg_match("/wp-admin/",$request_uri)) {
		return $type['default'];
	}
	/* If the client isn't providing an accept string */
	if(empty($accept_string)) {
		/* Then we can send whatever we like */
		return $type['xhtml'];
	}
	/* If the client is IE6 or greater, or pocket IE */
	if((preg_match('/msie/i',$user_agent) && !preg_match('/opera/i',$user_agent))
			||  preg_match('/mspie/i',$user_agent) || preg_match('/pocket/i', $user_agent)) {
		/* Then just feed it HTML */
		return $type['default'];
	}

	/* Checking for media type preference */
	foreach($type as $k=>$t)
	{
		if(strpos($accept_string,$t) === false) {
			$qs[$k] = 0;
		}
		else {
			$qs[$k] = q_value($t,$accept_string);
		}
	}

	/* If they prefer xhtml give them xhtml */
	if($qs['xhtml'] >= $qs['default'] || $qs['any'] > $qs['default']) {
		return $type['xhtml'];
	}
	/* Otherwise */
	return $type['default'];
}

add_filter('option_html_type','set_content_type')                                                             
?>
